@@ -93,11 +93,8 @@
 #define REQ_REFRESH 53
 #define REQ_SERVER_STATS 54
 #define REQ_CLIENT_ACCESSES_BY_INDEX2 55
-#define N_REQUEST_TYPES 56
-
-/* Special utoken value used to log on with first exchange being the
-   password.  (This time value has long since gone by) */
-#define SPECIAL_UTOKEN 0x10101010
+#define REQ_LOCAL2 56
+#define N_REQUEST_TYPES 57
 
 /* Structure used to exchange timevals independent on size of time_t */
 typedef struct {
@@ -215,6 +212,8 @@ typedef struct {
 typedef struct {
   int32_t on_off;
   int32_t stratum;
+  Float distance;
+  int32_t orphan;
   int32_t EOR;
 } REQ_Local;
 
