@@ -37,11 +37,23 @@
 #define SOCK_MAGIC 0x534f434b
 
 struct sock_sample {
+  /* Time of the measurement (system time) */
   struct timeval tv;
+
+  /* Offset between the true time and the system time (in seconds) */
   double offset;
+
+  /* Non-zero if the sample is from a PPS signal, i.e. another source
+     is needed to obtain seconds */
   int pulse;
+
+  /* 0 - normal, 1 - insert leap second, 2 - delete leap second */
   int leap;
+
+  /* Padding, ignored */
   int _pad;
+
+  /* Protocol identifier (0x534f434b) */
   int magic;
 };
 
